# Campuscard Mobile App

This is a Campuscard Mobile Application for mobile phones. Uses [Ionic](https://ionicframework.com/) with [Anguar](https://angular.io/) as a framework. 

We are using semantic versioning on this project.

## Prepare environment

1. Install [Node and npm](https://nodejs.org/en/download/current). We are currently using node v16.13.0 and npm 8.1.0.
2. Install Ionic: `npm install -g @ionic/cli cordova-res`. Current version: 6.18.1.

## After cloning from git

`npm install` (`npm i`) will install all dependencies.

## Running app locally (on localhost)

`ionic serve` will serve the app locally (some device-specific features may not work!).

## Running on Android

To generate the android app and copy all necessary assets, run the `build_android` script in the project directory.

If you have already generated a working android file before (via build_android) and want to update your build, just run `ionic cap copy`. This will just re-compile the source code without touching the custom assets (icons etc.) in the android file. (Running the script every time will also work.)

## Running with the provided backend

There is an application, edssi-kms, which is intended as a backend for this version of the app.
If you're looking at this text, you should also have the code and documentation for edssi-kms.
Once you have built and deployed the backend, you will need to modify the app's source code
so that it connects to it.
You can do this by editing the file `src/app/data/unis/index.ts`.
There is an URL in that file which ends with `/api/`.
Replace everything before `/api/` with the URL of your instance of the backend.
In the usual case (with the backend running on a Tomcat) the URL you end up with will look like
`http://192.168.1.10:8080/edssi/api/`.
Of course, you may also have some more complicated network configuration, but then you should be
able to work out the URL yourself.

Here's a brief explanation for how that URL gets used: there is an Angular interceptor,
`src/app/interceptors/api.interceptor.ts`, which modifies every request so that a URL stored
in the WebView's storage is prepended to the path.
The URL in `src/app/data/unis/index.ts` is stored in the relevant location
when the login page is presented.
More specifically, `src/app/pages/login/login.page.ts` calls a function in
`src/app/pages/login/login-facade.service.ts` which then sets the value.

# Capabilities

All university-dependent views, texts and other values are only displayed after login.

<img src="docs/screenshots/card.png" height="400">
<img src="docs/screenshots/esc.png" height="400">
<img src="docs/screenshots/features.png" height="400">
<img src="docs/screenshots/support.png" height="400">

* General
  + Uni Login and persistent JWT storing
  + Logout
  + App-wide translations
* My Card tab
  + Photo upload function
  + Optical representation of the card
  + Cache for card data; expires after 24h
  + European Student Card Features (ESC-R QR-Code + “Hologram”)
* Support tab
  + Q&A
  + Customized support emails with student data template
  + Data security alert for opening the email template

## What is Ionic

[Ionic](https://ionicframework.com/) is a UI framework to build mobile-like web-based experiences. It is cross platform: targets both Android and iOS UI systems. With help of other tools, like Ionic Capacitor, this mobile-like static single-page-application can be converted into a natively installable package for mobile phones and may also be published in Google Play and Apple's App Store.  

Any Ionic project uses [Angular](https://angular.io/), [React](https://reactjs.org/) or [Vue](https://vuejs.org/). That said, any Ionic developer, in addition to HTML and JavaScript must also be familiar with at least one of them and its peculiarities, respectively.

At Campuscard we have chosen Angular because our previous project ([KMS Date Trader](https://agnes.hu-berlin.de/kmsdt/)) was developed with it and we already had (positive) experience with it.

## Angular Basics

Angular is an [advanced](https://angular.io/guide/architecture) front-end framework made by Google for building client-side applications. It is written with TypeScript and it is modular: every Angular app has a *root* module and so-called *feature* modules. There are two types of feature modules:
- *view components*, providing screen elements for your application,
- *services*, that may be injected into components, providing further functionality not directly related to views.

What about UI? Angular comes with a UI system from Google called [Angular Material](https://material.angular.io/). Angular Material implements [Material Design components](https://material.io/components) for Angular. One can, of course, not use the Angular Material UI toolkit at all and
- use some other UI frameworks compatible with Angular or
- libraries providing just single Angular-compatible components (like a map or a chart components)
- use some HTML/CSS UI framework and adapt it to be compatible with Angular (encapsulate the raw HTML elements with Angular template system) - this one usually means lots of work.

Using Angular usually results in heavier (in terms of size on a disk, bytes needed for initial page load etc.) Websites than with other modern web frameworks, but also provides many features out-of-the-box. Bear in mind, because of the static nature of the websites without server side rendering (SSR), they can be cached quite well. Given our experiences with KMS Date Trader, in our applications improving the UX should actually be focused on improving the performance of the backend. For example, the KMS Date Trader got a slower over time, but not because of the problems with Angular itself, but because its DB got bigger and the requests were taking longer and longer. 

Angular also comes with [out-of-the-box support for reactive programming](https://angular.io/guide/rx-library) (rxjs). This allows to write Views (view components) and ViewModels fairly easy in a MVVM paradigm. In this project, every *page* is a view, every *facade* is a ViewModel.

## Similarities and differences between Ionic projects and regular websites

Despite many overlaps, the Ionic project differs in some aspects from a regular webpage. It will be running on a mobile device, so some additional assumptions can be made. For example, instead of a favicon for a regular website, the mobile app should have an icon, that will be rendered in the mobile phone's app drawer. Instead of using web storage APIs like [LocalStorage](https://developer.mozilla.org/pl/docs/Web/API/Window/localStorage) and [IndexedDB](https://developer.mozilla.org/en-US/docs/Web/API/IndexedDB_API), the app could use the mobile phone's file system to store space-demanding data, like photos. Ionic framework fortunately provides the developers with APIs for that, like [Ionic Storage](https://ionicframework.com/docs/angular/storage).

### Building the app: Cordova vs Capacitor

Shipping the app means copying the static HTML, JS and CSS files and wrapping them within a deployable unit, such as an APK file for Android. Ionic Framework (the UI framework) historically used [Apache Cordova](https://cordova.apache.org/) to ship apps for multiple platforms. Cordova, first released in 2009, is already quite old and has its limitations, for example using the browser's global scope or lacking in TypeScript sources. The Ionic team developed their own alternative to Cordova called [Capacitor](http://capacitorjs.com/). Capacitor offers shipping the app as an Android, iOS or PWA app. 

### Support for native APIs in Capacitor

Capacitor supports many native (Android and iOS) APIs with so-called [Capacitor Plugins](https://capacitorjs.com/docs/v3/basics/using-plugins). There are official and community-made plugins. There are also Enterprise plugins that have to be purhased from the Ionic team. The examples for official Capacitor Plugins include Push Notification, Camera, Geolocation, Clipboard and so on.

### APIs that are not supported out-of-the-box

Not all native APIs are supported by Ionic out-of-the-box. Fortunately, all of them can be supported with one's own plugins. The exemplary NFC Host Card Emulation plugin to emulate HU library card can be found in the [campuscard-hce-plugin repo](https://bb-git.zedat.fu-berlin.de/CampusCard/campuscard-hce-plugin) and see implemented in the [experiments/campuscard-hce-plugin](https://bb-git.zedat.fu-berlin.de/CampusCard/campuscard-mobile-app/-/tree/experiments/campuscard-hce-plugin) branch. To create a new Ionic Plugin, see [this](https://bb-git.zedat.fu-berlin.de/CampusCard/campuscard-hce-plugin).

## Default Ionic templates, internal routing and app's state

Ionic, similarly to Xcode and Android Studio, provides its users with a set of so-called [Starter Templates](https://github.com/ionic-team/starters). This app was [created](https://ionicframework.com/docs/cli/commands/start) using the `tabs` template and the [`ion-tabs`](https://ionicframework.com/docs/api/tabs) component is a central part of its internal routing structure. Unlike the default version of the program, the `ion-tabs` component is not placed on the root page, but it is visible only after login (in the logged-in state of the app). In theory, it should also be possible to implement somehow "dynamic" tabs and show or hide them depending on the app's state.

### Angular Router
For the whole application to work properly, hiding the tabs is not sufficient. Certain views and functions (paths in the routing structure) simply can not be accessible in some illegal states. [Angular Router](https://angular.io/api/router) gives the developers ability to implement conditional routing and also [prevent access to certain paths](https://angular.io/api/router/CanActivate). All contents available for logged-in users is protected using the [`LoggedInGuard`](src/app/logged-in-guard.ts) in the [`tabs-routing.module.ts`](src/app/pages/tabs/tabs-routing.module.ts) file. Please note that the return value of the `canActivate` function is checked only when the [RouterEvent](https://angular.io/api/router/RouterEvent) is fired. The guard returns a simple boolean and no [observable value](https://angular.io/guide/observables), because routing is a synchronous operation. 

### State: Ionic Storage, components' stores
The app's current route is certainly an important part of its internal state, but there are many other variables that have to be stored in order to make the app useful for the user. Some of them require persistence, other may only be needed temporarily, when the app is running.

Persistence is achieved with Ionic Storage, implemented as a [Storage](src/app/storage/storage-service.service.ts) service, a regular Angular service. Storage service takes care of initializing the Ionic Storage, initializing variables and providing other components with convenient methods for value manipulation and both synchronous (immediate) and asynchronous (as an observable) getters.

Oftentimes, when getting any `data` from backend, not only the specified `data` is (persistently) updated, but the component's `component-facade.service.ts` uses the `component-store.service.ts` class to handle the updates of the `dataUpdateInProgress` and `dataUpdateFailed` variables - they are used in the view layer to display information for the user, if `data` is being updated (for example using a [spinner](https://ionicframework.com/docs/api/spinner)) or there was a problem during update (error message). The usage of `dataUpdateInProgress` and `dataUpdateFailed` variables can probably be simplified using a more generic type of some kind, but was left like this for brevity - there aren't that many different endpoints in the app yet.

## Building, testing and publishing Ionic projects

To ensure high quality of any release, a CI/CD pipeline must be used. Ionic Framework promotes its own CI/CD tool called [Appflow](https://ionic.io/appflow), that can automatically build and test any new code pushed to the repo and - if it passed the tests - push the update to the app stores. Instead of Appflow, any other CI/CD tools can also be used, but the application's deployment (upload to the store, at least in the beta channel) should also be automated. The current trend is to host and orchestrate the CI/CD pipelines [directly from the version control system](https://docs.gitlab.com/ee/ci/) like Git.

There are two options to deploy a backend (or multiple backends) for testing and production purposes:
- backend infrastructure may be build and deployed together with the (client/mobile) app,
- backend infrastructure may be build and deployed separately.

This repo contains only client-side code, so we will further discuss only the second option. It is quite common that the test and prod builds come from just one codebase, but the different flavours of the application are generated using the env variables. The test and deployment pipelines must be able to generate both variants. As the (test-) backend is deployed separately, its accesibility from the test environment must also be taken into consideration. Optimally, the test infrastructure should resemble the production infrastructure as much possible. For example - if the primary use case of the app requires it to be available from the Internet, the test infrastructure should follow and the integration tests should embrace the properties of the target system. More about testing can be found in the [Ionic](https://ionicframework.com/docs/angular/testing) and [Angular](https://angular.io/guide/testing) Testing Guide.