import { Component } from '@angular/core';

import { Platform } from '@ionic/angular';
import { SplashScreen } from '@ionic-native/splash-screen/ngx';
import { StatusBar } from '@ionic-native/status-bar/ngx';
import { StorageService } from './storage/storage-service.service';
import { AllUnisCardBack, AllUnisCardFront } from '@campuscard/card-webcomponent/dist/card-all-unis';
import { TranslateService } from '@ngx-translate/core';
import { getDefaultAppLang } from 'src/assets/i18n/languages';

@Component({
  selector: 'app-root',
  templateUrl: 'app.component.html',
  styleUrls: ['app.component.scss']
})
export class AppComponent {
  constructor(
    private platform: Platform,
    private splashScreen: SplashScreen,
    private statusBar: StatusBar,
    private storage: StorageService,
    private translate: TranslateService,
  ) {
    this.initializeApp();
  }

  initializeApp() {
    const platformReady = this.platform.ready();
    const storageReady = this.storage.ready;
    const languageReady = this.setupAppLanguages();

    Promise.allSettled([platformReady, storageReady, languageReady])
      .then(() => {
        this.statusBar.styleDefault();
        this.splashScreen.hide();
      });

    this.defineIfNotDefined('x-card-front', AllUnisCardFront);
    this.defineIfNotDefined('x-card-back', AllUnisCardBack);
  }

  async setupAppLanguages() {
    // firstly, a default language is set, that will be used if no other language is forced
    const browserLang = this.translate.getBrowserLang();
    this.translate.setDefaultLang(getDefaultAppLang(browserLang));

    // then we check if another forced language is saved in the storage/ settings 
    const forcedLanguage = await this.storage.getAppLanguage();
    if (forcedLanguage) {
      this.translate.use(forcedLanguage);
    } else {
      this.translate.use(this.translate.defaultLang);
    }
  }

  defineIfNotDefined(name: string, constructor: CustomElementConstructor) {
    if (!customElements.get(name))
      customElements.define(name, constructor);
  }
}
