import { Injectable } from '@angular/core';
import {
  HttpRequest,
  HttpHandler,
  HttpEvent,
  HttpInterceptor
} from '@angular/common/http';
import { StorageService } from '../storage/storage-service.service';
import { Observable } from 'rxjs';

@Injectable()
export class ApiInterceptor implements HttpInterceptor {

  private uniApiUrlPrefix: string;

  constructor(private storage: StorageService) {
    this.storage.select(state => state.SELECTED_UNI).subscribe(uni => {
      if (uni) {
        this.uniApiUrlPrefix = uni.apiUrl;
      } else {
        this.uniApiUrlPrefix = undefined;
      }
    });
  }

  intercept(request: HttpRequest<any>, next: HttpHandler): Observable<HttpEvent<any>> {

    if (this.uniApiUrlPrefix) {
      // request will be augmented only if the uni is selected
      request = request.clone({
        url: this.uniApiUrlPrefix + request.url
      });
    }

    return next.handle(request);
  }
}
