import { Injectable } from '@angular/core';
import {
  HttpRequest,
  HttpHandler,
  HttpEvent,
  HttpInterceptor
} from '@angular/common/http';
import { StorageService } from '../storage/storage-service.service';
import { Observable } from 'rxjs';

@Injectable()
export class AuthInterceptor implements HttpInterceptor {

  private apiToken: string;

  constructor(private storage: StorageService) {
    this.storage.select(state => state.API_TOKEN).subscribe(token => this.apiToken = token);
  }

  intercept(request: HttpRequest<any>, next: HttpHandler): Observable<HttpEvent<any>> {

    if (this.apiToken) {
      // request will be augmented only if the token exists
      request = request.clone({
        setHeaders: {
          Authorization: `Bearer ${this.apiToken}`
        }
      });
    }
    return next.handle(request);
  }
}