import { Injectable } from '@angular/core';
import { KmsApi } from 'src/app/apis/kms.service';
import { LoginStore } from './login-store.service';
import { StorageService } from 'src/app/storage/storage-service.service';
import { Router } from '@angular/router';
import { Uni } from 'src/app/types';

@Injectable({
  providedIn: 'root'
})
export class LoginFacade {

  constructor(private store: LoginStore, private storage: StorageService, private kmsApi: KmsApi, private router: Router) { }

  getUni() {
    return this.storage.select(state => state.SELECTED_UNI);
  } 

  selectUni(uni: Uni) {
    this.storage.setSelectedUni(uni);
  }

  isLoginFailed() {
    return this.store.isLoginFailed();
  }

  isLoginInProgress() {
    return this.store.isLoginInProgress();
  }

  ionViewWillEnter() {
    this.store.setLoginFailed(false);
    this.store.setLoginInProgress(false);

    this.storage.select(state => state.API_TOKEN).subscribe(apiToken => {
      if (apiToken) this.router.navigate(['menu']);
    });
  }

  onBackButtonClick() {
    this.storage.removeSelectedUni();
  }

  login(username: string, password: string) {
    this.store.setLoginFailed(false);
    this.store.setLoginInProgress(true);

    const requestBody = {
      username: username,
      password: password
    }

    this.kmsApi.login(requestBody)
      .then(response => {
        this.storage.setApiToken(response.idToken);
      })
      .catch(() => this.store.setLoginFailed(true))
      .finally(() => this.store.setLoginInProgress(false))
  }
}
