import { Component, OnInit } from '@angular/core';
import { FormControl, FormGroup } from '@angular/forms';
import { ViewWillEnter } from '@ionic/angular';
import { Observable } from 'rxjs';
import { EXAMPLE } from 'src/app/data/unis';
import { Uni } from 'src/app/types';
import { LoginFacade } from './login-facade.service';

@Component({
  selector: 'app-login',
  templateUrl: './login.page.html',
  styleUrls: ['./login.page.scss'],
})
export class LoginPage implements OnInit, ViewWillEnter {

  loginFormGroup: FormGroup;
  uni: Observable<Uni>;
  isLoginFailed: Observable<boolean>;
  isLoginInProgress: Observable<boolean>;

  constructor(private facade: LoginFacade) {
    this.isLoginFailed = this.facade.isLoginFailed();
    this.isLoginInProgress = this.facade.isLoginInProgress();
    this.uni = this.facade.getUni();
  }

  ngOnInit() {
    this.loginFormGroup = new FormGroup({
      username: new FormControl(''),
      password: new FormControl(''),
    });
  }

  ionViewWillEnter() {
    this.facade.selectUni(EXAMPLE);
    this.facade.ionViewWillEnter();
  }

  onBackButtonClick() {
    this.facade.onBackButtonClick();
  }

  onSubmit() {
    const username = this.loginFormGroup.value.username;
    const password = this.loginFormGroup.value.password;
    this.facade.login(username, password);
  }

}
