import { Injectable } from '@angular/core';
import { StorageService } from 'src/app/storage/storage-service.service';
import { KmsApi } from 'src/app/apis/kms.service';
import { EscStore } from './esc-store.service';

@Injectable({
  providedIn: 'root'
})
export class EscFacade {

  constructor(private storage: StorageService, private kmsApi: KmsApi, private store: EscStore) { }

  getIsEscEnabled() { return this.storage.select(state => state.IS_ESC_ENABLED) };
  getEscIdentifier() { return this.storage.select(state => state.ESC_IDENTIFIER) };
  getEscNumber() { return this.storage.select(state => state.ESC_NUMBER) };

  async loadEscData(): Promise<string> {
    this.store.setEscDataUpdateFailed(false);
    this.store.setEscDataUpdateInProgress(true);
    try {
      const isEscEnabled = await this.kmsApi.isEscEnabled();
      if (isEscEnabled) {
        Promise.allSettled([this.loadEscIdentifier(), this.loadEscIdentifier()]);
        return await this.storage.setIsEscEnabled(true);
      } else {
        return await this.storage.setIsEscEnabled(false);
      }
    } catch {
      this.store.setEscDataUpdateFailed(true);
    } finally {
      this.store.setEscDataUpdateInProgress(false);
    }
  }

  async loadEscIdentifier(): Promise<string> {
    try {
      const escIdentifier = await this.kmsApi.getEscIdentifier();
      return await this.storage.setEscIdentifier(escIdentifier);
    } catch {
      // currently, it is not specified whether the Number or Identifier failed when loading
    } finally {
      // currently, it is not specified whether the Number or Identifier failed when loading
    }
  }

  async loadEscNumber(): Promise<string> {
    try {
      const escNumber = await this.kmsApi.getEscNumber();
      return await this.storage.setEscNumber(escNumber);
    } catch {
      // currently, it is not specified whether the Number or Identifier failed when loading
    } finally {
      // currently, it is not specified whether the Number or Identifier failed when loading
    }
  }
  
}
