import { Injectable } from '@angular/core';
import { LibraryStore } from './library-store.service';
import { StorageService } from 'src/app/storage/storage-service.service';
import { HCEService } from 'src/app/services/hce.service';
import { TranslateService } from '@ngx-translate/core';

@Injectable({
  providedIn: 'root'
})
export class LibraryFacade {
  hceToggleOnText: string;
  hceToggleOffText: string;

  constructor(
    private store: LibraryStore, 
    private storage: StorageService,
    private hceService: HCEService,
    private translateService: TranslateService) {}

  isHceActive() { return this.storage.select(state => state.HCE_IS_ACTIVE); }
  isHceToggleOn() { return this.store.isHceToggleOn(); }

  // Translations for HCE section:
  getTranslations() {
    this.translateService.get('library').subscribe(library => {
      this.hceToggleOnText = library.toggleOn;
      this.hceToggleOffText = library.toggleOff;
    })
  }

  async setHce(hceOn: boolean) {
    this.store.setHceToggleOn(hceOn);
    this.hceService.setHceStatus(hceOn);
  }

}
