import { Injectable } from '@angular/core';
import { IonRefresher } from '@ionic/angular';
import { TranslateService } from '@ngx-translate/core';
import { KmsApi } from 'src/app/apis/kms.service';
import { PhotoService } from 'src/app/services/photo.service';
import { StorageService } from 'src/app/storage/storage-service.service';
import { CardData } from 'src/app/types';
import { MyCardStore } from './my-card-store.service'

@Injectable({
  providedIn: 'root'
})
export class MyCardFacade {
  
  constructor(private storage: StorageService,
    private store: MyCardStore,
    private kmsApi: KmsApi,
    private photoService: PhotoService,
    private translateService: TranslateService) { }

  getSelectedUni() { return this.storage.select(state => state.SELECTED_UNI); }

  getCardData() { return this.storage.select(state => state.CARD_DATA); }
  isCardDataUpdateFailed() { return this.store.isCardDataUpdateFailed(); }
  isCardDataUpdateInProgress() { return this.store.isCardDataUpdateInProgress(); }
  isCardBlocked() { return this.storage.select(state => state.CARD_IS_BLOCKED); }
  getPhoto() {return this.photoService.getPhoto(); }
  getEscIdentifier() { return this.storage.select(state => state.ESC_IDENTIFIER); }
  getLastCardDataFetchDate() { return this.store.getLastCardDataFetchDate(); }

  // Drag-Refresher:
  doRefresh(refresher?: IonRefresher) {
    this.loadCardData();
    this.cardBlockedCheck();
    refresher?.complete()
  }

  private async loadCardData(): Promise<CardData> {
    this.store.setCardDataUpdateFailed(false);
    this.store.setCardDataUpdateInProgress(true);

    try {
      const cardData = await this.kmsApi.getCard();
      // If loading is successful, update the fetch date for caching:
      this.store.setLastCardDataFetchDate(new Date());
      return await this.storage.setCardData(cardData);
    } catch {
      this.store.setCardDataUpdateFailed(true);
    } finally {
      this.store.setCardDataUpdateInProgress(false);
    }
  }

  async loadEscData(): Promise<string> {
    try {
      const escIdentifier = await this.kmsApi.getEscIdentifier();
      return await this.storage.setEscIdentifier(escIdentifier);
    } catch {
      // currently, we do not indicate if the esc data failed when loading
    } finally {
      // currently, we do not indicate if the esc data failed is done loading 
    }
  }

  async cardBlockedCheck(): Promise<boolean> {
    try {
      const isCardBlocked = await this.kmsApi.isCardBlocked();
      return await this.storage.setIsCardBlocked(isCardBlocked);
    } catch {
      // currently, we do not indicate if the card blocked data failed when loading
    } finally {
      // currently, we do not indicate if the card blocked data failed when loading
    }
  }

}
