import { Injectable } from "@angular/core";
import { PhotoUploadStore } from "./photoupload-store.service";
import { Camera, CameraResultType, CameraSource } from '@capacitor/camera';
import { KmsApi } from 'src/app/apis/kms.service';
import { StorageService } from 'src/app/storage/storage-service.service';
import { PhotoService } from "src/app/services/photo.service";

@Injectable({
  providedIn: 'root'
})
export class PhotoUploadFacade {
  constructor(private kmsApi: KmsApi,
    private storage: StorageService,
    private store: PhotoUploadStore,
    private photoService: PhotoService) { }

  isPhotoUploadInProgress() { return this.store.isPhotoUploadInProgress(); }
  isPhotoUploadFailed() { return this.store.isPhotoUploadFailed(); }
  isServerError() { return this.store.isServerError(); }
  getSelectedUni() { return this.storage.select(state => state.SELECTED_UNI); }
  getPhoto() { return this.photoService.getPhoto(); }
  getCardData() { return this.storage.select(state => state.CARD_DATA); }

  setInitialValues() {
    this.store.setPhotoUploadFailed(false);
    this.store.setServerError(false);
  }

  takePhoto() {
    this.storePhoto(CameraSource.Camera);
  }

  selectPhoto() {
    this.storePhoto(CameraSource.Photos);
  }

  private async storePhoto(source) {
    this.setInitialValues();

    const capturedPhoto = await Camera.getPhoto({
      quality: 100,
      resultType: CameraResultType.DataUrl,
      source: source,
    });

    var canvas = document.createElement('canvas'),
      MAX_WIDTH = 400,
      MAX_HEIGHT = 600,
      img = new Image(),
      imgBase64 = "";

    img.src = capturedPhoto.dataUrl;
    await img.decode();

    var width = img.width;
    var height = img.height;

    if (width > height) {
      if (width > MAX_WIDTH) {
        height *= MAX_WIDTH / width;
        width = MAX_WIDTH;
      }
    } else {
      if (height > MAX_HEIGHT) {
        width *= MAX_HEIGHT / height;
        height = MAX_HEIGHT;
      }
    }

    canvas.width = width;
    canvas.height = height;

    var ctx = canvas.getContext('2d');
    ctx.drawImage(img, 0, 0, img.width, img.height, 0, 0, width, height);

    imgBase64 = canvas.toDataURL('image/jpeg', 0.5).substring(23);

    this.store.setPhotoUploadInProgress(true);
    this.kmsApi.uploadPhoto(imgBase64)
      .then(response => this.photoService.storePhoto(response))
      .catch(error => {
        this.store.setPhotoUploadFailed(true);
        if (error.status > 499) {
         this.store.setServerError(true);
        }
      })
      .finally(() => this.store.setPhotoUploadInProgress(false));
  }

}



