import { Component } from '@angular/core';
import { ViewWillEnter } from '@ionic/angular';
import { Observable } from 'rxjs';
import { PhotoUploadFacade } from './photoupload-facade.service';
import { Uni, CardData } from 'src/app/types';
import { MenuPage } from '../menu.page';
import { NavigationService } from 'src/app/services/navigation.service';

@Component({
  selector: 'app-photoupload',
  templateUrl: './photoupload.page.html',
  styleUrls: ['./photoupload.page.scss'],
})
export class PhotouploadPage implements ViewWillEnter {

  photo: Observable<string | null>;
  photoObject: string | null;
  isPhotoUploadFailed: Observable<boolean>;
  isPhotoUploadInProgress: Observable<boolean>;
  isServerError: Observable<boolean>;
  selectedUni: Observable<Uni>;
  cardData: Observable<CardData>;
  
  constructor(private facade: PhotoUploadFacade, public menuPage: MenuPage, public navigation: NavigationService) {
    this.photo = this.facade.getPhoto();
    this.photo.subscribe(photo => 
      this.photoObject = photo);
    this.isPhotoUploadFailed = this.facade.isPhotoUploadFailed();
    this.isPhotoUploadInProgress = this.facade.isPhotoUploadInProgress();
    this.isServerError = this.facade.isServerError();
    this.selectedUni = this.facade.getSelectedUni();
    this.cardData = this.facade.getCardData();
  }

  ionViewWillEnter(): void {
    this.facade.setInitialValues();
  }

  takePhoto() {
    this.facade.takePhoto();
  };

  selectPhoto() {
    this.facade.selectPhoto();
  }

  back() {
    this.navigation.back();
  }
  
}
