import { Injectable } from '@angular/core';
import { Router } from '@angular/router';
import { AlertOptions } from '@ionic/core';
import { TranslateService } from '@ngx-translate/core';
import { StorageService } from 'src/app/storage/storage-service.service';
import { HCEService } from 'src/app/services/hce.service';
import { LANGUAGES } from 'src/assets/i18n/languages';

@Injectable({
  providedIn: 'root'
})
export class SettingsFacade {

  constructor(
    private storage: StorageService, 
    private router: Router, 
    private translate: TranslateService, 
    private hceService: HCEService
    ) { }

  getAppLanguage() {
    return this.storage.select(state => state.APP_LANGUAGE);
  }

  getForceAppLanguageAlert(alreadySavedAppLanguageKey: string): AlertOptions {
    let alertNewValue = <string>undefined;
    return {
      header: this.translate.instant('settings.forceLanguage.header'),
      inputs: [
        {
          type: 'radio',
          // TODO: think about translating the label
          label: 'Automatic',
          checked: !alreadySavedAppLanguageKey,
          handler: () => alertNewValue = null
        },
        ...LANGUAGES.map(language => ({
          type: <const>"radio",
          label: language.label,
          checked: language.key === alreadySavedAppLanguageKey,
          handler: () => alertNewValue = language.key
        })),
      ],
      buttons: [
        {
          text: this.translate.instant('common.cancel'),
        }, {
          text: this.translate.instant('common.ok'),
          handler: () => {
            if (alertNewValue !== undefined) {
              this.setAppLanguage(alertNewValue);
            }
          }
        }
      ]
    }
  }

  setAppLanguage(languageKey: string) {
    if (languageKey) {
      this.storage.setAppLanguage(languageKey)
        .then(() => this.translate.use(languageKey))
    } else {
      this.storage.removeAppLanguage()
        .then(() => {
          const defaultLanguage = this.translate.getDefaultLang();
          this.translate.use(defaultLanguage);
        });
    }
  }

  logout() {
    this.storage.reset();
    this.hceService.deleteLibId();
    this.router.navigate(['login']);
  }

  async getVersion() {
    return import('package.json').then(packageJson => packageJson.version);
  }
}
