import { Component } from '@angular/core';
import { AlertController } from '@ionic/angular';
import { BehaviorSubject } from 'rxjs';
import { SettingsFacade } from './settings-facade.service';
import { NavigationService } from 'src/app/services/navigation.service';
import { MenuPage } from '../menu.page';

@Component({
  selector: 'app-settings',
  templateUrl: './settings.page.html',
  styleUrls: ['./settings.page.scss'],
})
export class SettingsPage {

  appLanguage = new BehaviorSubject<string>(undefined);
  version = new BehaviorSubject("");

  constructor(private facade: SettingsFacade, 
    public navigation: NavigationService, 
    private alertController: AlertController, 
    public menuPage: MenuPage) {
    this.facade.getAppLanguage().subscribe(value => this.appLanguage.next(value));
    this.facade.getVersion().then(version => this.version.next(version));
  }

  back(): void {
    this.navigation.back();
  }

  logout() {
    this.facade.logout();
  }

  async presentAlertRadio() {
    const alertOptions = this.facade.getForceAppLanguageAlert(this.appLanguage.value);
    const alert = await this.alertController.create(alertOptions);
    await alert.present();
  }

  getVersion() {
    return this.facade.getVersion();
  }

}
