import { Injectable, OnDestroy } from '@angular/core';
import { AlertController } from '@ionic/angular';
import { TranslateService } from '@ngx-translate/core';
import { Observable, Subscription } from 'rxjs';
import { StorageService } from 'src/app/storage/storage-service.service';
import { CardData } from 'src/app/types';

// Email line break:
const BREAK = "%0D%0A";

@Injectable({
  providedIn: 'root'
})
export class SupportFacade implements OnDestroy {

  alertHeader: string;
  alertContent: string;
  alertYes: string;
  alertNo: string;

  cardData: Observable<CardData>;
  cardDataSubscription: Subscription;
  name: string;
  uni: string;
  uniId: string;
  uniSubscription: Subscription;
  birthday: string;
  studentId: string;

  supportMail: string;
  mailSubject: string;
  mailBody: string;
  birthdayText: string;
  studentIdText: string;
  studentDataMailTemplate: string;

  constructor(private storage: StorageService, private translateService: TranslateService, private alertController: AlertController) { 
    this.cardData = this.getCardData();
    this.uniSubscription = this.getUni().subscribe(uni => {
      this.uni = uni.name;
      this.uniId = uni.id;
    });
    this.cardDataSubscription = this.cardData.subscribe(data => {
      this.name = data.name;
      this.birthday = data.birthday;
      this.studentId = data.studentId;
    });
    this.supportMail = "hello@example.com"
  }

  // Data for email body:
  getUni() { return this.storage.select(state => state.SELECTED_UNI); }
  getCardData() { return this.storage.select(state => state.CARD_DATA); }

  // Translations for data security alert and email template:
  getTranslations() {
    this.translateService.get('support').subscribe(support => {
      // Path to translation entries:
      const path = support.entries.example.email;
      // Alert:
      this.alertHeader = path.alertHeader;
      this.alertContent = path.alertContent;
      this.alertYes = path.alertYes;
      this.alertNo = path.alertNo;
      // Mail template:
      this.mailSubject = path.mailSubject;
      this.mailBody = path.mailBody;
      this.birthdayText = path.birthday;
      this.studentIdText = path.studentId;
      this.studentDataMailTemplate = [this.name, this.uni, (this.studentIdText + this.studentId), (this.birthdayText + this.birthday)].join(BREAK);
    })
  }

  // Data security alert, forwarding to external email template:
  async presentAlertConfirm() {
    const alert = await this.alertController.create({
      header: this.alertHeader,
      message: this.alertContent,
      buttons: [
        {
          text: this.alertNo,
          role: 'cancel',
          handler: () => {
          }
        }, {
          text: this.alertYes,
          handler: () => {
              window.location.href = this.mailToWithTemplate(this.supportMail);
          }
        }
      ]
    });
    await alert.present();
  }

  mailToWithTemplate(address: string): string {
    return ["mailto:", address, "?subject=", this.mailSubject, "&body=", this.mailBody].join('') + this.studentDataMailTemplate;
  }

  ngOnDestroy(): void {
    this.uniSubscription.unsubscribe();
    this.cardDataSubscription.unsubscribe();
  }

}
