import { Injectable } from '@angular/core';
import { Emulation } from '@campuscard/hce-plugin';
import { StorageService } from 'src/app/storage/storage-service.service';

// This service provides the libraryId from the App's cardData to the HCE-Plugin, which then writes it into a Desfire-Application.
// Currently, there is just this one HCE application for reading the libraryId.

@Injectable({
    providedIn: 'root'
  })
  export class HCEService {
    constructor(private storage: StorageService) {}

    libId: string;

    getCardData() { return this.storage.select(state => state.CARD_DATA); };
    
    insertLibId(): void {
        this.getCardData().subscribe(data => this.libId = data.libraryId);
        Emulation.changeAppData({ data: this.libId });
    }

    deleteLibId(): void {
        Emulation.changeAppData({ data: "" });
    }

    setHceStatus(status: boolean): void {
      this.storage.setIsHceActive(status);
      if (status === true) {
        this.insertLibId();
      } else {
        this.deleteLibId();
      }
    }

  }