// Service for wrapping the back navigation. 
// Listening to router events of type NavigationEnd to manage an app-specific navigation history.
// If the history still contains entries after popping the current URL off of the stack, we can safely navigate back. 
// - otherwise, we're falling back to the application root.

import { Injectable } from '@angular/core'
import { Router, NavigationEnd } from '@angular/router'
import { Location } from '@angular/common'

@Injectable({ providedIn: 'root' })
export class NavigationService {
  private history: string[] = []

  constructor(private router: Router, private location: Location) {
    this.router.events.subscribe((event) => {
      if (event instanceof NavigationEnd) {
        this.history.push(event.urlAfterRedirects)
      }
    })
  }

  back(): void {
    this.history.pop()
    if (this.history.length > 0) {
      this.location.back()
    } else {
      this.router.navigateByUrl('/')
    }
  }
}