import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { filter, map } from 'rxjs/operators';
import { StorageService } from '../storage/storage-service.service';

@Injectable({
  providedIn: 'root'
})
export class PhotoService {
  constructor(private storage: StorageService) { }

  /** This just expects the base64 string; you don't need to prepend anything. */
  storePhoto(photoString: string): Promise<any> {
    return this.storage.getCardData().then(cardData => {
      cardData.photo = photoString;
      return this.storage.setCardData(cardData);
    })
  }

  /** This fetches the base64 string from storage and prepends the magic string that
   * says it's a jpeg; the result is displayable.
   */
  getPhoto(): Observable<string | null> {
    var photoOrNull = this.storage.select(state => state.CARD_DATA)
      .pipe(map(cardData => cardData?.photo ? 'data:image/jpg;base64,' + cardData.photo : null));
    return photoOrNull;
  }
}
