import { Injectable } from '@angular/core';
import { Storage } from '@ionic/storage';
import { CardData, Uni } from '../types';
import { Store } from './abstract.store';
import { CoreState } from './core.state';

@Injectable({
  providedIn: 'root'
})
export class StorageService extends Store<CoreState>{

  private readonly _prefix = 'campuscard';
  private readonly appLanguage = 'APP_LANGUAGE';
  private readonly selectedUni = 'SELECTED_UNI';
  private readonly apiToken = 'API_TOKEN';
  private readonly cardData = 'CARD_DATA';
  private readonly isEscEnabled = 'IS_ESC_ENABLED';
  private readonly escNumber = 'ESC_NUMBER';
  private readonly escIdentifier = 'ESC_IDENTIFIER';
  private readonly cardIsBlocked = 'CARD_IS_BLOCKED';
  private readonly hceIsActive = 'HCE_IS_ACTIVE';
  public ready: Promise<any>;

  constructor(private storage: Storage) {
    super(new CoreState());

    this.ready = Promise.all([
      this.initValue(this.appLanguage),
      this.initValue(this.selectedUni),
      this.initValue(this.apiToken),
      this.initValue(this.cardData),
      this.initValue(this.isEscEnabled),
      this.initValue(this.escNumber),
      this.initValue(this.escIdentifier),
      this.initValue(this.cardIsBlocked),
      this.initValue(this.hceIsActive)
    ]);
  }

  private initValue(prop: keyof CoreState): Promise<any> {
    const key = `${this._prefix}-${prop}`;
    return this.storage.get(key).then(value => {
      this.setStateValue(prop, value);
      return value;
    }).catch(_ => {
      this.setStateValue(prop, undefined);
      return undefined;
    });
  }

  private setStateValue(prop: keyof CoreState, value: any): void {
    this.state = {
      ...this.state,
      [prop]: value
    };
  }

  private setValue(prop: keyof CoreState, value: any): Promise<any> {
    this.setStateValue(prop, value);
    const key = `${this._prefix}-${prop}`;
    return this.storage.set(key, value);
  }

  private readValue(prop: keyof CoreState): Promise<any> {
    const key = `${this._prefix}-${prop}`;
    return this.storage.get(key);
  }

  private removeValue(prop: keyof CoreState): Promise<any> {
    this.setStateValue(prop, undefined);
    const key = `${this._prefix}-${prop}`;
    return this.storage.remove(key);
  }

  public reset() {
    return Promise.allSettled([
      this.removeAppLanguage(),
      this.removeSelectedUni(),
      this.removeApiToken(),
      this.removeCardData(),
      this.removeIsEscEnabled(),
      this.removeEscNumber(),
      this.removeEscIdentifier(),
      this.removeCardIsBlocked(),
      this.removeHceIsActive()
    ]);
  }

  public setAppLanguage(value: string): Promise<any> { return this.setValue(this.appLanguage, value); }
  public getAppLanguage(): Promise<string> { return this.readValue(this.appLanguage); }
  public removeAppLanguage(): Promise<any> { return this.removeValue(this.appLanguage); }

  public setSelectedUni(value: Uni): Promise<any> { return this.setValue(this.selectedUni, value); }
  public getSelectedUni(): Promise<Uni> { return this.readValue(this.selectedUni); }
  public removeSelectedUni(): Promise<any> { return this.removeValue(this.selectedUni); }

  public setApiToken(value: string): Promise<any> { return this.setValue(this.apiToken, value); }
  public getApiToken(): Promise<string> { return this.readValue(this.apiToken); }
  public removeApiToken(): Promise<any> { return this.removeValue(this.apiToken); }

  public setCardData(value: CardData): Promise<any> { return this.setValue(this.cardData, value); }
  public getCardData(): Promise<CardData> { return this.readValue(this.cardData); }
  public removeCardData(): Promise<any> { return this.removeValue(this.cardData); }

  public setIsEscEnabled(value: boolean): Promise<any> { return this.setValue(this.isEscEnabled, value); }
  public getIsEscEnabled(): Promise<boolean> { return this.readValue(this.isEscEnabled); }
  public removeIsEscEnabled(): Promise<any> { return this.removeValue(this.isEscEnabled); }

  public setEscNumber(value: string): Promise<any> { return this.setValue(this.escNumber, value); }
  public getEscNumber(): Promise<string> { return this.readValue(this.escNumber); }
  public removeEscNumber(): Promise<any> { return this.removeValue(this.escNumber); }

  public setEscIdentifier(value: string): Promise<any> { return this.setValue(this.escIdentifier, value); }
  public getEscIdentifier(): Promise<string> { return this.readValue(this.escIdentifier); }
  public removeEscIdentifier(): Promise<any> { return this.removeValue(this.escIdentifier); }

  public setIsCardBlocked(value: boolean): Promise<any> { return this.setValue(this.cardIsBlocked, value); }
  public getCardIsBlocked(): Promise<boolean> { return this.readValue(this.cardIsBlocked); }
  public removeCardIsBlocked(): Promise<any> { return this.removeValue(this.cardIsBlocked); }

  public setIsHceActive(value: boolean): Promise<any> { return this.setValue(this.hceIsActive, value); }
  public getHceIsActive(): Promise<boolean> { return this.readValue(this.hceIsActive); }
  public removeHceIsActive(): Promise<any> { return this.removeValue(this.hceIsActive); }
}
